summaryrefslogtreecommitdiff
path: root/block/io.c
diff options
context:
space:
mode:
authorEric Blake <eblake@redhat.com>2017-07-07 07:44:59 -0500
committerKevin Wolf <kwolf@redhat.com>2017-07-10 13:18:07 +0200
commit51b0a488882328f8f02519bb47ca7e0e7fbe12ff (patch)
tree36257cf2221ea7e4e54ab7deae446d02c46f326c /block/io.c
parentc00716beb30ba996bd6fdfd5f41bb07e4414144f (diff)
downloadqemu-51b0a488882328f8f02519bb47ca7e0e7fbe12ff.tar.gz
block: Make bdrv_is_allocated_above() byte-based
We are gradually moving away from sector-based interfaces, towards byte-based. In the common case, allocation is unlikely to ever use values that are not naturally sector-aligned, but it is possible that byte-based values will let us be more precise about allocation at the end of an unaligned file that can do byte-based access. Changing the signature of the function to use int64_t *pnum ensures that the compiler enforces that all callers are updated. For now, the io.c layer still assert()s that all callers are sector-aligned, but that can be relaxed when a later patch implements byte-based block status. Therefore, for the most part this patch is just the addition of scaling at the callers followed by inverse scaling at bdrv_is_allocated(). But some code, particularly stream_run(), gets a lot simpler because it no longer has to mess with sectors. Leave comments where we can further simplify by switching to byte-based iterations, once later patches eliminate the need for sector-aligned operations. For ease of review, bdrv_is_allocated() was tackled separately. Signed-off-by: Eric Blake <eblake@redhat.com> Signed-off-by: Kevin Wolf <kwolf@redhat.com>
Diffstat (limited to 'block/io.c')
-rw-r--r--block/io.c38
1 files changed, 18 insertions, 20 deletions
diff --git a/block/io.c b/block/io.c
index aad7fd30a4..23170a57ee 100644
--- a/block/io.c
+++ b/block/io.c
@@ -1926,50 +1926,48 @@ int coroutine_fn bdrv_is_allocated(BlockDriverState *bs, int64_t offset,
/*
* Given an image chain: ... -> [BASE] -> [INTER1] -> [INTER2] -> [TOP]
*
* Return true if the given sector is allocated in any image between
* BASE and TOP (inclusive). BASE can be NULL to check if the given
* sector is allocated in any image of the chain. Return false otherwise,
* or negative errno on failure.
*
- * 'pnum' is set to the number of sectors (including and immediately following
- * the specified sector) that are known to be in the same
- * allocated/unallocated state.
+ * 'pnum' is set to the number of bytes (including and immediately
+ * following the specified offset) that are known to be in the same
+ * allocated/unallocated state. Note that a subsequent call starting
+ * at 'offset + *pnum' may return the same allocation status (in other
+ * words, the result is not necessarily the maximum possible range);
+ * but 'pnum' will only be 0 when end of file is reached.
*
*/
int bdrv_is_allocated_above(BlockDriverState *top,
BlockDriverState *base,
- int64_t sector_num,
- int nb_sectors, int *pnum)
+ int64_t offset, int64_t bytes, int64_t *pnum)
{
BlockDriverState *intermediate;
- int ret, n = nb_sectors;
+ int ret;
+ int64_t n = bytes;
intermediate = top;
while (intermediate && intermediate != base) {
int64_t pnum_inter;
int64_t size_inter;
- int psectors_inter;
- ret = bdrv_is_allocated(intermediate, sector_num * BDRV_SECTOR_SIZE,
- nb_sectors * BDRV_SECTOR_SIZE,
- &pnum_inter);
+ ret = bdrv_is_allocated(intermediate, offset, bytes, &pnum_inter);
if (ret < 0) {
return ret;
}
- assert(pnum_inter < INT_MAX * BDRV_SECTOR_SIZE);
- psectors_inter = pnum_inter >> BDRV_SECTOR_BITS;
if (ret) {
- *pnum = psectors_inter;
+ *pnum = pnum_inter;
return 1;
}
- size_inter = bdrv_nb_sectors(intermediate);
+ size_inter = bdrv_getlength(intermediate);
if (size_inter < 0) {
return size_inter;
}
- if (n > psectors_inter &&
- (intermediate == top || sector_num + psectors_inter < size_inter)) {
- n = psectors_inter;
+ if (n > pnum_inter &&
+ (intermediate == top || offset + pnum_inter < size_inter)) {
+ n = pnum_inter;
}
intermediate = backing_bs(intermediate);
@@ -1926,50 +1926,48 @@ int coroutine_fn bdrv_is_allocated(BlockDriverState *bs, int64_t offset,
* Given an image chain: ... -> [BASE] -> [INTER1] -> [INTER2] -> [TOP]
*
- * Return true if the given sector is allocated in any image between
- * BASE and TOP (inclusive). BASE can be NULL to check if the given
- * sector is allocated in any image of the chain. Return false otherwise,
+ * Return true if (a prefix of) the given range is allocated in any image
+ * between BASE and TOP (inclusive). BASE can be NULL to check if the given
+ * offset is allocated in any image of the chain. Return false otherwise,
* or negative errno on failure.
*